/****************************************************************************/
/*
 * MODULE              JN-AN-1170 RF-Remote Control
 *
 * DESCRIPTION         Capacitance Touch Remote Demo - Main Source File
 */
/****************************************************************************/
/*
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5148, JN5142, JN5139].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2012. All rights reserved
 */
/****************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/
#include <jendefs.h>
#include <AppHardwareApi.h>
#include "DriverCapTouch.h"
#include "Printf.h"

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
#define CPU_CLOCK_HZ (16000000)
#define IIR_FILTER_HZ     (100)

#define LHS_LED_DIO        (0)
#define RHS_LED_DIO        (1)
#define LED_OFF  vAHI_DioSetOutput( ((1<<LHS_LED_DIO) | (1<<RHS_LED_DIO)),0)
#define LED_ON   vAHI_DioSetOutput(0, ((1<<LHS_LED_DIO) | (1<<RHS_LED_DIO)))

#define DIO_WAKEMASK	(1<<LHS_LED_DIO)

#define INACTIVITY_TIME_S 	 10
#define TICKS_PER_SECOND 	100
#define SLEEP_LED_PULSE_TIME (TICKS_PER_SECOND/2)

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
typedef enum
{
	E_TIMER_RESET,
	E_TIMER_TICK
} teTimerControl;
/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/

PRIVATE void vInitSystem(void);

PRIVATE void vTickTimerISR(uint32 u32DeviceId, uint32 u32ItemBitmap);

PRIVATE  void vSysConISR(uint32 u32DeviceId, uint32 u32ItemBitmap);

PRIVATE void vManageInactivityTimer(teTimerControl eTimerControl);

/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

PRIVATE volatile uint32 u32Tick = 0;
PRIVATE volatile bool_t bWakeEvent = FALSE;

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/
/****************************************************************************
 *
 * NAME: AppColdStart
 *
 * DESCRIPTION:
 * Entry point for application from boot loader. Initialises system and runs
 * main loop.
 *
 * RETURNS:
 * Never returns.
 *
 ****************************************************************************/
PUBLIC void AppColdStart(void)
{

    vInitSystem();

	vPrintf("\n***************************************************");
	vPrintf("\n* Cap Touch Remote Demo (Press any Key on Remote) *");
	vPrintf("\n***************************************************");

	while(1)
	{
		vAHI_CpuDoze();

		if (u32Tick !=0) /* consume the 100 Hz ticks outside interrupt context */
		{
			u32Tick = 0;
			eTouchProcess();
			vManageInactivityTimer(E_TIMER_TICK);
		}
	}
}

/****************************************************************************
 *
 * NAME: AppWarmStart
 *
 * DESCRIPTION:
 * Entry point for application from boot loader. Simply jumps to AppColdStart
 * as, in this instance, application will never warm start.
 *
 * RETURNS:
 * Never returns.
 *
 ****************************************************************************/
PUBLIC void AppWarmStart(void)
{
    AppColdStart();
}

/****************************************************************************
 *
 * NAME: vCbTouchEventButton
 *
 * DESCRIPTION:
 * Callback Handler function for the Cap Touch Interface.  Will be called
 * when an touch event occurs. This can be one of:
 *
 * a) PRESSED[BUTTON_ID]
 * b) RELEASED[BUTTON_ID]
 * c) ALL RELEASED
 *
 ****************************************************************************/

PUBLIC void vCbTouchEventButton(eTouchButtonEvent eEvent, uint8 u8ButtonNumber)
{
	static const char *apcEventStrings[] = {"RELEASED", "PRESSED", "ALL_RELEASED"};

	static const char *pcKeySymbols ="#D|O42CB13-+A*?>";

	vPrintf("\nTOUCH_BUTTON_EVENT_%s : ButtonID = %d [%c]",apcEventStrings[eEvent],
			                                               u8ButtonNumber,
			                                               pcKeySymbols[u8ButtonNumber]);

	/* Turn on leds when any pad touched, off when no pads touched */

	if (eEvent == TOUCH_BUTTON_EVENT_PRESSED)
	{
		LED_ON;
	}
	else if (eEvent == TOUCH_BUTTON_EVENT_RELEASED)
	{
		LED_OFF;
	}

	/*reset the inactivity timer when an event occurs */

	vManageInactivityTimer(E_TIMER_RESET);
}

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/

/****************************************************************************
 *
 * NAME: vInitSystem
 *
 * DESCRIPTION:
 *
 * Sets up the hardware and starts the capacitance touch interface
 *
 ****************************************************************************/
PRIVATE void vInitSystem(void)
{
	vAHI_SysCtrlRegisterCallback(vSysConISR);
    /* Initialise the Hardware Interfaces */
    (void)u32AHI_Init();

    /* Disable watchdog if enabled by default */
#ifdef WATCHDOG_ENABLED
    vAHI_WatchdogStop();
#endif

    /* Set Tick timer for 100 Hz Interrupts */
    vAHI_TickTimerConfigure(E_AHI_TICK_TIMER_DISABLE);
    vAHI_TickTimerInterval(CPU_CLOCK_HZ/IIR_FILTER_HZ);
    vAHI_TickTimerRegisterCallback(vTickTimerISR);
    vAHI_TickTimerIntEnable(TRUE);
    vAHI_TickTimerConfigure(E_AHI_TICK_TIMER_RESTART);

    /* Initialise LED DIO */
    vAHI_DioSetDirection(0,((1<<LHS_LED_DIO) | (1<<RHS_LED_DIO)));
    vAHI_DioSetOutput(((1<<LHS_LED_DIO) | (1<<RHS_LED_DIO)),0);


    vUART_printInit();
    vAHI_UartSetRTSCTS(E_AHI_UART_0, FALSE);

    if (bWakeEvent)
    {
    	bWakeEvent = FALSE;
    	vPrintf("\nWoke Up");
    }
    eTouchInit();

    while(eTouchProcess() ==  TOUCH_STATUS_DONT_SLEEP);

}

/****************************************************************************
 *
 * NAME: Interrupt Service Routines (ISRs)
 *
 * DESCRIPTION:
 *
 * Handlers for the tick timer and system controller (DIO) interrupts
 *
 ****************************************************************************/

PRIVATE void vTickTimerISR(uint32 u32DeviceId, uint32 u32ItemBitmap)
{
	u32Tick = 1;
}

PRIVATE void vSysConISR(uint32 u32DeviceId, uint32 u32ItemBitmap)
{
	if (u32DeviceId == E_AHI_DEVICE_SYSCTRL)
	{
		if (u32ItemBitmap & E_AHI_DIO0_INT)
		{
			bWakeEvent = TRUE;
		}
	}
}

/****************************************************************************
 *
 * NAME: vManageInactivityTimer
 *
 * DESCRIPTION:
 *
 * Puts the Device to sleep after a period of inactivity
 *
 ****************************************************************************/
PRIVATE void vManageInactivityTimer(teTimerControl eTimerControl)
{
	static uint32 u32InactivityTimer = INACTIVITY_TIME_S * TICKS_PER_SECOND;

	if (eTimerControl == E_TIMER_RESET)
	{
		u32InactivityTimer = INACTIVITY_TIME_S * TICKS_PER_SECOND;
	}
	else if (eTimerControl == E_TIMER_TICK)
	{
		if (u32InactivityTimer == SLEEP_LED_PULSE_TIME)
		{
			vAHI_DioSetOutput(0,  (1<<RHS_LED_DIO));   /* RHS LED ON */
		}

		if (--u32InactivityTimer == 0)
		{
			vPrintf("\nInactivity Timeout: Sleeping");

			//let any debug out
			while ((u8AHI_UartReadLineStatus(E_AHI_UART_0) & E_AHI_UART_LS_THRE) == 0);
			while ((u8AHI_UartReadLineStatus(E_AHI_UART_0) & E_AHI_UART_LS_TEMT) == 0);

			/* Disable UART & Tick Timer */
			vAHI_UartDisable(E_AHI_UART_0);
			vAHI_TickTimerIntEnable(FALSE);

			/* Reconfigure LHS led to act as wake input */
		    vAHI_DioSetDirection(DIO_WAKEMASK,0);
		    vAHI_DioSetPullup(DIO_WAKEMASK,0);
            vAHI_DioWakeEdge(0,DIO_WAKEMASK);
		    vAHI_DioWakeEnable(DIO_WAKEMASK,0);

		    vAHI_DioSetOutput((1<<RHS_LED_DIO),0);   /* RHS LED off */
		    vAHI_Sleep(E_AHI_SLEEP_DEEP);

		}
	}
}
/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
